@@ -49,19 +49,19 @@ DEFAULT_PORTS = {'t':'50001', 's':'50002', 'h':'8081', 'g':'8082'}
 DEFAULT_SERVERS = {
     'erbium1.sytes.net':{'t':'50001', 's':'50002'},
     'ecdsa.net':{'t':'50001', 's':'110'},
-    'ELECTRUM.top-master.com':{'t':'50001', 's':'50002'},
+    'gh05.geekhosters.com':{'t':'50001', 's':'50002'},
     'VPS.hsmiths.com':{'t':'50001', 's':'50002'},
-    'ELECTRUM.jdubya.info':{'t':'50001', 's':'50002'},
+    'electrum.anduck.net':{'t':'50001', 's':'50002'},
     'electrum.no-ip.org':{'t':'50001', 's':'50002', 'g':'443'},
     'us.electrum.be':DEFAULT_PORTS,
-    'bitcoins.sk':{'t':'50001', 's':'50002'},
-    'us1.einfachmalnettsein.de':{'t':'50001', 's':'50002'},
-    'electrum.dragonzone.net':DEFAULT_PORTS,
+    'electrum.villocq.com':{'t':'50001', 's':'50002'},
+    'us10.einfachmalnettsein.de':{'t':'50001', 's':'50002'},
+    'electrum.trouth.net':{'t':'50001', 's':'50002'},
     'Electrum.hsmiths.com':{'t':'8080', 's':'995'},
     'electrum3.hachre.de':{'t':'50001', 's':'50002'},
     'elec.luggs.co':{'t':'80', 's':'443'},
     'btc.smsys.me':{'t':'110', 's':'995'},
-    'electrum.online':{'t':'50001', 's':'50002'},
+    'btc.mustyoshi.com':{'t':'50001', 's':'50002'},
 }
 
 NODES_RETRY_INTERVAL = 60
