@@ -35,6 +35,7 @@ from electrum import bitcoin, util
 from electrum import transaction
 from electrum.plugins import BasePlugin, hook
 from electrum.i18n import _
+from electrum.wallet import Multisig_Wallet
 
 from electrum_gui.qt.transaction_dialog import show_transaction
 
@@ -43,7 +44,7 @@ import traceback
 
 
 PORT = 12344
-HOST = 'ecdsa.net'
+HOST = 'cosigner.electrum.org'
 server = xmlrpclib.ServerProxy('http://%s:%d'%(HOST,PORT), allow_none=True)
 
 
@@ -114,7 +115,7 @@ class Plugin(BasePlugin):
 
     def update(self, window):
         wallet = window.wallet
-        if wallet.wallet_type not in ['2of2', '2of3']:
+        if type(wallet) != Multisig_Wallet:
             return
         if self.listener is None:
             self.print_error("starting listener")
@@ -130,7 +131,7 @@ class Plugin(BasePlugin):
             xpub = keystore.get_master_public_key()
             K = bitcoin.deserialize_xkey(xpub)[-1].encode('hex')
             _hash = bitcoin.Hash(K).encode('hex')
-            if wallet.master_private_keys.get(key):
+            if not keystore.is_watching_only():
                 self.keys.append((key, _hash, window))
             else:
                 self.cosigner_list.append((window, xpub, K, _hash))
@@ -189,7 +190,7 @@ class Plugin(BasePlugin):
             return
 
         wallet = window.wallet
-        if wallet.use_encryption:
+        if wallet.has_password():
             password = window.password_dialog('An encrypted transaction was retrieved from cosigning pool.\nPlease enter your password to decrypt it.')
             if not password:
                 return
@@ -198,7 +199,7 @@ class Plugin(BasePlugin):
             if not window.question(_("An encrypted transaction was retrieved from cosigning pool.\nDo you want to open it now?")):
                 return
 
-        xprv = wallet.get_master_private_key(key, password)
+        xprv = wallet.keystore.get_master_private_key(password)
         if not xprv:
             return
         try:
